/* Copyright (c) 2000-2008 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

var ko, prefContainer, prefContainerType;
// prefContainer is one of a koIPart, koIDocument, or null (for global)
// Unfortunately the attribute names for common operations like getting
// file and preference objects for projectParts and documents take
// different names (e.g. "koIFileEx getFile()" vs.
// "readonly attribute koIFileEx file").

// fields

var sleuth_language;

// This line maps field name to required status

var text_field_names = {sleuth_name:[1, 0],
                        sleuth_dirname:[1, 2],
                        sleuth_command_line:[0, 3]};
var dialog_fields = {};

function doBrowseForDir() {
    // This is a bit coarse -- better to suggest either the
    // current dir, or if there is none, the project's; and
    // then if they cancel out don't put anything in the field.
    // The problem here is we'll start in Komodo's default directory.
    ko.filepicker.browseForDir(dialog_fields.sleuth_dirname);
    return true;
}

function createTestPlanCancel() {
    return true;
}

function createTestPlanOK() {
    var labels = document.getElementsByTagName("label");
    var missing_field = null;
    var missing_names = [];
    for (var name in text_field_names) {
        // labels[text_field_names[name][1]].style.color = 'black';
        if (text_field_names[name][0] && !dialog_fields[name].value) {
            var label;
            var labelNode = document.getElementById(dialog_fields[name] + "_label");
            if (labelNode) {
                label = labelNode.getAttribute("label");
            } else {
                label = name;
            }
            missing_names.push(label);
            if (!missing_field) {
                missing_field = dialog_fields[name];
            }
            //labels[text_field_names[name][1]].style.color = 'red';
        }
    }
    if (missing_names.length > 0) {
        alert("The following fields cannot be left blank: "
              + missing_names.join(", "));
        missing_field.focus();
        return false;
    }
    
    var gSleuthPrefs = null;
    var globalPrefs = Components.classes["@activestate.com/koPrefService;1"].getService(Components.interfaces.koIPrefService).prefs;
    var prefset;
    var container_name;
    switch(prefContainerType) {
      case 'project':
       prefset = (prefContainer ? prefContainer.prefset : globalPrefs);
       container_name = 'The current project "' + prefContainer.name + '"';
       break;
      case 'document':
       prefset = (prefContainer ? prefContainer.prefs : globalPrefs);
       container_name = 'The current file "' + prefContainer.displayPath + '"';
       break;
      default:
       prefset = globalPrefs;
       container_name = "The global preference-set";
    }
    const TESTPLAN_PREFNAME = (Components.classes["@activestate.com/koSleuthService;1"].
                               getService(Components.interfaces.koISleuthService).TESTPLAN_PREFNAME);
    if (prefset.hasPrefHere(TESTPLAN_PREFNAME)) {
        gSleuthPrefs = prefset.getPref(TESTPLAN_PREFNAME);
    } else {
        gSleuthPrefs = Components.classes["@activestate.com/koPreferenceSet;1"].createInstance();
        prefset.setPref(TESTPLAN_PREFNAME, gSleuthPrefs);
    }

    var name = dialog_fields['sleuth_name'].value;
    if (gSleuthPrefs.hasPrefHere(name)) {
        alert(container_name
              + " already has a test plan named \""
              + name
              + "\"");
        return false;
    }
    
    var testPlanPref = Components.classes["@activestate.com/koPreferenceSet;1"].createInstance();
    testPlanPref.setStringPref('command_line',
                               dialog_fields.sleuth_command_line.value);
    ko.sleuth.manager.setPrefToDir(testPlanPref,
                                   dialog_fields.sleuth_dirname.value);
    testPlanPref.setStringPref('language',
                               document.getElementById("sleuth_language").label);
    gSleuthPrefs.setPref(name, testPlanPref);
    // This doesn't chain into the project prefs notification service,
    // so trigger it manually.
    // Bug 74901: select this test-plan
    if (prefContainerType == 'project'
        && prefContainer
        && ko.projects.manager.getCurrentProject() != prefContainer) {
        ko.projects.manager.setCurrentProject(prefContainer);
    }
    try {
        var topic = ("select_"
                     + (prefContainerType || "global")
                     + "_testplan");
        globalPrefs.prefObserverService.notifyObservers(this, topic, name);
        ko.sleuth.tabManager.runTests(name,
                                      dialog_fields.sleuth_dirname.value,
                                      document.getElementById("sleuth_language").label,
                                      dialog_fields.sleuth_command_line.value);
    } catch(ex) {
    }
    return true;
}

function createTestPlanLoad() {
    ko = window.arguments[0];
    prefContainer = window.arguments[1];
    prefContainerType = window.arguments[2];
    var currentTestPlanName_label = document.getElementById("currentTestPlanName.label");
    currentTestPlanName_label.setAttribute("accesskey", "N");
    currentTestPlanName_label.setAttribute("tooltiptext",
                                           "Each test plan needs a name");
    var currentTestPlanName_textbox = document.getElementById("sleuth_name");
    currentTestPlanName_textbox.setAttribute("pref", true);
    for (var name in text_field_names) {
        dialog_fields[name] = document.getElementById(name);
    }
    sleuth_language = document.getElementById('sleuth_language');
                                           
    if (prefContainerType == 'project') {
        dialog_fields.sleuth_name.value = (prefContainer.name
                                           + " - test plan # "
                                           + (ko.sleuth.manager.getNumProjectTestPlans(prefContainer, 'prefset') + 1));
        dialog_fields.sleuth_dirname.value = prefContainer.getFile().dirName;
    } else if (prefContainerType == 'document') {
        dialog_fields.sleuth_name.value = (prefContainer.file.leafName
                                           + " - test plan # "
                                           + (ko.sleuth.manager.getNumProjectTestPlans(prefContainer, 'prefs') + 1));
        dialog_fields.sleuth_dirname.value = prefContainer.file.dirName;
    } else {
        dialog_fields.sleuth_name.value = ("global test plan # "
                                           + (ko.sleuth.manager.getNumGlobalTestPlans() + 1));
        try {
            dialog_fields.sleuth_dirname.value = ko.window.getCwd();
        } catch(ex) {
            // This shouldn't fail
            dump(ex + "\n");
            dialog_fields.sleuth_dirname.value = "";
        }
    }
    var harness_indices = {};
    ko.sleuth.manager.initHarnessPopupMenu(sleuth_language, harness_indices);
    // Figure out the default language (harness)
    var harness_info_from_language = {};
    ko.sleuth.manager.getLangInfoFromHarnessIndices(harness_indices, harness_info_from_language);
    var current_lang = ((prefContainerType != 'project')
                        ? _getCurrentLanguage()
                        : null);
    var idx = 0; // default: use the first entry in the menu, whatever it will be.
    if (current_lang && current_lang in harness_info_from_language) {
        idx = harness_info_from_language[current_lang].menuIdx;
    } else if (prefContainerType == 'project') {
        current_lang = (ko.sleuth.manager.getFirstRecognizedProjectLanguage(prefContainer, harness_info_from_language)
                        || _getCurrentLanguage());
        if (current_lang && current_lang in harness_info_from_language) {
            idx = harness_info_from_language[current_lang].menuIdx;
        }
    }
    var elt = document.getElementById("sleuth_language");
    elt.selectedIndex = idx;
}

function _getCurrentLanguage() {
    try {
        return ko.views.manager.currentView.koDoc.language;
    } catch(ex) {
        return null;
    }
}


